/**
 * 
 */
package net.bigeon.gclc.swt;

/*-
 * #%L
 * GCLC swt
 * %%
 * Copyright (C) 2015 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */

import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

import net.bigeon.gclc.utils.AOutputForwardRunnable;
import net.bigeon.gclc.utils.PipedConsoleOutput;

/** The manager for console output to insert in a text.
 * 
 * @author Emmanuel Bigeon */
public final class ConsoleOutputManager implements ConsoleOutputDisplay {
    /** A runnable appending text to the content of a {@link Text} component.
     * 
     * @author Emmanuel Bigeon */
    private static class TextAppendingRunnable implements Runnable {
        /** The text to append on a line (possibly new). */
        private final String next;
        /** The {@link Text} component. */
        private final Text   text;

        /** Create the appending runnable.
         * 
         * @param text the component to update
         * @param next the text to append */
        public TextAppendingRunnable(final Text text, final String next) {
            this.text = text;
            this.next = next;
        }

        /* (non-Javadoc)
         * @see java.lang.Runnable#run() */
        @Override
        public void run() {
            final String initialText = text.getText();
            if (initialText != null && !initialText.isEmpty()) {
                text.append(System.lineSeparator());
            }
            text.append(next);
        }
    }

    /** The local implementation of the forwarding runnable.
     *
     * @author Emmanuel Bigeon */
    private static final class ToSWTConsoleForwardRunnable
            extends AOutputForwardRunnable {
        /** The running status. */
        private boolean                    running = true;
        /** The console output. */
        private final PipedConsoleOutput   out;
        /** The console output display. */
        private final ConsoleOutputDisplay display;
        /** The actual SWT component. */
        private final Widget               element;

        /** Create the forwarding runnable.
         * 
         * @param manager the manager
         * @param display the display
         * @param element the composite */
        public ToSWTConsoleForwardRunnable(final PipedConsoleOutput manager,
                                           final ConsoleOutputDisplay display,
                                           final Widget element) {
            super(manager);
            out = manager;
            this.display = display;
            this.element = element;
        }

        /* (non-Javadoc)
         * @see
         * net.bigeon.gclc.utils.AOutputForwardRunnable#forwardLine(java.lang.String) */
        @Override
        protected void forwardLine(final String m) {
            display.appendLine(m);
        }

        /* (non-Javadoc)
         * @see net.bigeon.gclc.utils.AOutputForwardRunnable#isRunning() */
        @Override
        protected boolean isRunning() {
            return running && !element.isDisposed();
        }

        /** Set the running status.
         * 
         * @param running the running to set */
        public void setRunning(final boolean running) {
            this.running = running;
        }

        /** Get the output.
         * 
         * @return the currently forwarded output */
        public PipedConsoleOutput getOuput() {
            return out;
        }
    }

    /** The SWT component displaying the output content. */
    private final Text                  text;
    /** The forwarding runnable. */
    private ToSWTConsoleForwardRunnable forward;
    /** The forwarding thread. */
    private Thread                      forwardThread;

    /** Create the manager.
     * 
     * @param text the text to display the output in. */
    public ConsoleOutputManager(final Text text) {
        super();
        this.text = text;
    }

    /* (non-Javadoc)
     * @see net.bigeon.gclc.swt.ConsoleOutputDisplay#appendLine(java.lang.String) */
    @Override
    public void appendLine(final String next) {
        text.getDisplay().syncExec(new TextAppendingRunnable(text, next));
    }

    /** Set the output.
     *
     * @param output the output to set */
    public void setManager(final PipedConsoleOutput output) {
        if (forward != null) {
            if (forward.getOuput() == output) {
                return;
            }
            forward.setRunning(false);
        }
        if (output == null) {
            forward = null;
            forwardThread = null;
            return;
        }
        forward = new ToSWTConsoleForwardRunnable(output, this, text);
        forwardThread = new Thread(forward, "gclcToSWT"); //$NON-NLS-1$
        forwardThread.start();
    }

    /** Get the current thread forwarding the console output to the SWT components.
     * 
     * @return the forwardThread */
    public Thread getForwardThread() {
        return forwardThread;
    }
}
